/* This file is part of the Allegria project
   Copyright (C) 2004 Francois Schiettekatte <francois.schiettekatte@umontreal.ca>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// Line edit widget that allows the number input only in the xxx.xxe+xxx format 
// (Obsolete: Qt now has the QDoubleValidator class that does just that)

#include "FSDoubleValidator.h"

FSDoubleValidator::FSDoubleValidator(QObject * parent, double mmin, double mmax)
 :QValidator(parent) {

  min = mmin;
  max = mmax;

}

QValidator::State FSDoubleValidator::validate ( QString & str, int & pos ) const
{
    bool DotChar = false;
    bool ExpChar = false;
    bool FirstDigit = false;
    bool LastDigit = false;
    bool Sign = false;
    bool FirstSign = false;

    bool accept = true;
    int l = str.length();
    for(int i=0;i<l;i++) {
        switch (str.at(i).toAscii()) {
        case '+':
        case '-':
            if(!FirstSign && !FirstDigit)
                FirstSign = true;
            else if(!Sign && ExpChar && !LastDigit)
                Sign = true;
            else
                accept = false;
            break;
        case '0': case '1': case '2': case '3': case '4':
        case '5': case '6': case '7': case '8': case '9':
            if ( ExpChar ) LastDigit = true;
            else FirstDigit = true;
            break;
        case '.':
            if ( DotChar || ExpChar ) accept = false;
            else DotChar = true;
            break;
        case 'E':
        case 'e':
            if ( ExpChar || !FirstDigit ) accept = false;
            else ExpChar = true;
            break;
        default:
            accept = false;
        }
        if(!accept) 
          return QValidator::Invalid;
    }
    double n = str.toDouble();
    if(accept && n>=min && n<=max) return QValidator::Acceptable;

    return QValidator::Invalid;
}

